package com.example.application.data.service;

import com.example.application.data.chesscom.ChessComArchive;
import com.example.application.data.chesscom.ChessComArchiveList;
import com.example.application.data.chesscom.ChessComGame;
import com.example.application.data.entity.Game;
import com.example.application.data.entity.GameInfo;
import com.example.application.data.entity.Player;
import com.example.application.utils.ChessComUtils;
import com.example.application.utils.HttpUtils;
import com.google.gson.Gson;
import org.springframework.lang.NonNull;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.stream.Collectors;


@Service
public class ChessComService {
    // TODO: make everything nullsafe

    private static final Gson gson = new Gson();

    public ChessComService() {
    }

    @NonNull
    public List<Game> getLatestGamesBetweenPlayers(@NonNull Player player1, @NonNull Player player2, int minimum_amount_of_games) {
        String url = String.format("https://api.chess.com/pub/player/%s/games/archives", ChessComUtils.getUrlPart(player1));

        List<String> archiveUrls = gson.fromJson(HttpUtils.getJson(url), ChessComArchiveList.class).getArchives();
        archiveUrls.sort(Collections.reverseOrder());

        if (archiveUrls.size() >= 2) {
            archiveUrls = archiveUrls.subList(0, 2);
        }

        List<Game> list = new ArrayList<>();
        for (String archiveUrl : archiveUrls) {
            String unpreparedJson = HttpUtils.getJson(archiveUrl);
            if (unpreparedJson == null) {
                continue;
            }
            String preparedJson = ChessComUtils.getPreparedArchiveJson(unpreparedJson);
            List<ChessComGame> games = gson.fromJson(preparedJson, ChessComArchive.class).getGames();

            list.addAll(games.stream()
                    .sorted(Comparator.comparingLong(ChessComGame::getEndTime).reversed())
                    .filter(chessComGame -> ChessComUtils.isGameBetween(chessComGame, player1, player2))
                    .filter(ChessComUtils::hasValidTimeControl)
                    .map(chessComGame -> getGame(chessComGame, player1, player2))
                    .collect(Collectors.toList()));

            if (list.size() >= minimum_amount_of_games) {
                break;
            }
        }
        return list;
    } // TODO: find exactly two games of each time control

    @NonNull
    private Game getGame(@NonNull ChessComGame chessComGame, @NonNull Player player1, @NonNull Player player2) {
        Game game = new Game();
        GameInfo gameInfo = new GameInfo();

        game.setPlayer1IsWhite(chessComGame.getWhite().getUsername().equals(player1.getNickname()));
        game.setResult(ChessComUtils.getResult(chessComGame));
        game.setGameInfo(gameInfo);

        gameInfo.setUrl(chessComGame.getUrl());
        gameInfo.setFormat(Integer.parseInt(chessComGame.getTimeControl())/60); // TODO: change gameinfo.format in database to reflect this!
        gameInfo.setGame(game);

        return game; // important: match is not set here!
    }

}
